param(
[string]$PLAIN_TEXT
)


function New-IcsEvent
{
    [OutputType([System.IO.FileInfo[]])]
    [CmdletBinding(SupportsShouldProcess, ConfirmImpact = 'Low')]
    param
    (
        [Parameter(Mandatory, ValueFromPipeline, ValueFromPipelineByPropertyName)]
        [datetime[]]
        $StartDate,

        [Parameter(Mandatory)]
        [string]
        $Path,

        [Parameter(Mandatory)]
        [string]
        $Subject,

        [Parameter()]
        [string]
        $Location,

        [Parameter()]
        [timespan]
        $Duration = '01:00:00',

        [Parameter()]
        [string]
        $EventDescription,

        [Parameter()]
        [switch]
        $PassThru,

        [ValidateSet('Free', 'Busy')]
        [string]
        $ShowAs = 'Busy',

        [ValidateSet('Private', 'Public', 'Confidential')]
        [string]
        $Visibility = 'Public',

        [string[]]
        $Category
    )

    begin
    {
        # Custom date formats that we want to use
        $icsDateFormat = "yyyyMMddTHHmmssZ"
    }

    process
    {
        # Checkout RFC 5545 for more options
        foreach ($Date in $StartDate)
        {
            $fileName = Join-Path -Path $Path -ChildPath "$($Date.ToString($icsDateFormat)).ics"
            $event = @"
BEGIN:VCALENDAR
VERSION:2.0
METHOD:PUBLISH
PRODID:-//JHP//We love PowerShell!//EN
BEGIN:VEVENT
UID: $([guid]::NewGuid())
CREATED: $((Get-Date).ToUniversalTime().ToString($icsDateFormat))
DTSTAMP: $((Get-Date).ToUniversalTime().ToString($icsDateFormat))
LAST-MODIFIED: $((Get-Date).ToUniversalTime().ToString($icsDateFormat))
CLASS:$Visibility
CATEGORIES:$($Category -join ',')
SEQUENCE:0
DTSTART: $($Date.ToUniversalTime().ToString($icsDateFormat))
DTEND: $($Date.Add($Duration).ToUniversalTime().ToString($icsDateFormat))
DESCRIPTION: $EventDescription
SUMMARY: $Subject
LOCATION: $Location
TRANSP:$(if($ShowAs -eq 'Free') {'TRANSPARENT'})
END:VEVENT
END:VCALENDAR
"@
            if ($PSCmdlet.ShouldProcess($fileName, 'Write ICS file'))
            {
                $event | Out-File -FilePath $fileName -Encoding utf8 -Force
                if ($PassThru) { Get-Item -Path $fileName }
            }
        }
    }
}


try{
    Add-Type -Path $PSScriptRoot\Microsoft.Extensions.Caching.Abstractions.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Extensions.Caching.Memory.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Extensions.DependencyInjection.Abstractions.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Extensions.Options.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Extensions.Primitives.dll
    Add-Type -Path $PSScriptRoot\Newtonsoft.Json.dll
    Add-Type -Path $PSScriptRoot\System.Runtime.CompilerServices.Unsafe.dll
    Add-Type -Path $PSScriptRoot\System.ValueTuple.dll
    Add-Type -Path $PSScriptRoot\System.Collections.Immutable.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.Number.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.NumberWithUnit.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Definitions.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.DateTime.dll
    Add-Type -Path $PSScriptRoot\DateTimeV2Converter.dll
    
}
catch [System.Reflection.ReflectionTypeLoadException]
{
  Write-Host "Message: $($_.Exception.Message)" -ForegroundColor Green
  Write-Host "StackTrace: $($_.Exception.StackTrace)" -ForegroundColor Yellow
  Write-Host "LoaderExceptions: $($_.Exception.LoaderExceptions)" -ForegroundColor Cyan
}


$culture = (Get-Culture).TwoLetterISOLanguageName


$time = New-Object DateTime

try{
    $time = [DateTimeV2Converter.Converter]::RecogniseDateTime($PLAIN_TEXT, $culture)
}
catch{
    $time = $null
}

$time_string = ""
if($time -eq $null){
    $time_string = Get-Date -Format "yyyy-MM-dd HH:mm"
}
else {

    $time_string = $time.ToString("yyyy-MM-dd HH:mm")
}


# Either create events one by one 
$event = (New-IcsEvent -Path $env:TEMP -Subject $PLAIN_TEXT -StartDate $time_string -ShowAs Free -PassThru).FullName

Invoke-Item $event
